@@ -574,6 +574,7 @@ def complete_xontrib(prefix, line, start, end, ctx):
 #
 # _aliases
 #
+inspect = _LazyModule.load('inspect', 'inspect')
 # amalgamated builtins
 collections = _LazyModule.load('collections', 'collections')
 # amalgamated xonsh.lazyasd
@@ -647,19 +648,29 @@ def _register_completer(args, stdin=None):
                "For help, run:  completer help add")
     else:
         name = args[0]
-        func = args[1]
+        func_name = args[1]
         if name in builtins.__xonsh_completers__:
             err = ("The name %s is already a registered "
                    "completer function.") % name
         else:
-            if func in builtins.__xonsh_ctx__:
-                if not callable(builtins.__xonsh_ctx__[func]):
-                    err = "%s is not callable" % func
+            if func_name in builtins.__xonsh_ctx__:
+                func = builtins.__xonsh_ctx__[func_name]
+                if not callable(func):
+                    err = "%s is not callable" % func_name
             else:
-                err = "No such function: %s" % func
+                print(inspect.stack(context=0))
+                for frame_info in inspect.stack(context=0):
+                    frame = frame_info[0]
+                    if func_name in frame.f_locals:
+                        func = frame.f_locals[func_name]
+                        break
+                    elif func_name in frame.f_globals:
+                        func = frame.f_globals[func_name]
+                        break
+                else:
+                    err = "No such function: %s" % func_name
     if err is None:
         position = "start" if len(args) == 2 else args[2]
-        func = builtins.__xonsh_ctx__[func]
         _add_one_completer(name, func, position)
     else:
         return None, err + '\n', 1
@@ -994,7 +1005,7 @@ def _quote_to_use(x):
         return single
 
 
-def _quote_paths(paths, start, end):
+def _quote_paths(paths, start, end, append_end=True):
     expand_path = builtins.__xonsh_expand_path__
     out = set()
     space = ' '
@@ -1030,7 +1041,8 @@ def _quote_paths(paths, start, end):
                 s += backslash
         if end in s:
             s = s.replace(end, ''.join('\\%s' % i for i in end))
-        out.add(start + s + end)
+        s = start + s + end if append_end else start + s
+        out.add(s)
     return out
 
 
@@ -1063,8 +1075,11 @@ def _splitpath_helper(path, sofar=()):
     folder, path = os.path.split(path)
     if path:
         sofar = sofar + (path, )
-    if (not folder or folder == xt.get_sep() or
-       (xp.ON_WINDOWS and os.path.splitdrive(path)[0])):
+    if not folder or folder == xt.get_sep():
+        return sofar[::-1]
+    elif xp.ON_WINDOWS and not path:
+        return os.path.splitdrive(folder)[:1] + sofar[::-1]
+    elif xp.ON_WINDOWS and os.path.splitdrive(path)[0]:
         return sofar[::-1]
     return _splitpath_helper(folder, sofar)
 
@@ -1114,6 +1129,7 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
     # string stuff for automatic quoting
     path_str_start = ''
     path_str_end = ''
+    append_end = True
     p = _path_from_partial_string(line, end)
     lprefix = len(prefix)
     if p is not None:
@@ -1121,6 +1137,8 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
         prefix = p[1]
         path_str_start = p[2]
         path_str_end = p[3]
+        if len(line) >= end + 1 and line[end] == path_str_end:
+            append_end = False
     tilde = '~'
     paths = set()
     env = builtins.__xonsh_env__
@@ -1160,7 +1178,8 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
     paths = set(filter(filtfunc, paths))
     paths = _quote_paths({_normpath(s) for s in paths},
                          path_str_start,
-                         path_str_end)
+                         path_str_end,
+                         append_end)
     paths.update(filter(filtfunc, _dots(prefix)))
     paths.update(filter(filtfunc, _env(prefix)))
     return paths, lprefix
@@ -1176,7 +1195,7 @@ def complete_dir(prefix, line, start, end, ctx, cdpath=False):
 """Completers for Python code"""
 # amalgamated re
 # amalgamated sys
-inspect = _LazyModule.load('inspect', 'inspect')
+# amalgamated inspect
 # amalgamated builtins
 importlib = _LazyModule.load('importlib', 'importlib')
 cabc = _LazyModule.load('collections', 'collections.abc', 'cabc')
@@ -1266,7 +1285,7 @@ def complete_python_mode(prefix, line, start, end, ctx):
     if not (prefix.startswith('@(') or prefix.startswith('${')):
         return set()
     prefix_start = prefix[:2]
-    python_matches = complete_python(prefix[2:], line, start+2, end, ctx)
+    python_matches = complete_python(prefix[2:], line, start-2, end-2, ctx)
     if isinstance(python_matches, cabc.Sequence):
         python_matches = python_matches[0]
     return set(prefix_start + i for i in python_matches)
