@@ -10,16 +10,16 @@ from prompt_toolkit.shortcuts import print_tokens
 from prompt_toolkit.styles import PygmentsStyle, style_from_dict
 
 from xonsh.base_shell import BaseShell
-from xonsh.tools import print_exception
+from xonsh.tools import print_exception, carriage_return
 from xonsh.ptk.completer import PromptToolkitCompleter
 from xonsh.ptk.history import PromptToolkitHistory
 from xonsh.ptk.key_bindings import load_xonsh_bindings
 from xonsh.ptk.shortcuts import Prompter
 from xonsh.events import events
 from xonsh.shell import transform_command
-from xonsh.platform import HAS_PYGMENTS
+from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS
 from xonsh.style_tools import partial_color_tokenize, _TokenType, DEFAULT_STYLE_DICT
-from xonsh.lazyimps import pygments, pyghooks
+from xonsh.lazyimps import pygments, pyghooks, winutils
 
 Token = _TokenType()
 
@@ -36,9 +36,12 @@ class PromptToolkitShell(BaseShell):
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
+        if ON_WINDOWS:
+            winutils.enable_virtual_terminal_processing()
+        self._first_prompt = True
         self.prompter = Prompter()
         self.history = PromptToolkitHistory()
-        self.pt_completer = PromptToolkitCompleter(self.completer, self.ctx)
+        self.pt_completer = PromptToolkitCompleter(self.completer, self.ctx, self)
         key_bindings_manager_args = {
             'enable_auto_suggest_bindings': True,
             'enable_search': True,
@@ -153,7 +156,7 @@ class PromptToolkitShell(BaseShell):
                 else:
                     line = self.precmd(line)
                     self.default(line)
-            except KeyboardInterrupt:
+            except (KeyboardInterrupt, SystemExit):
                 self.reset_buffer()
             except EOFError:
                 if builtins.__xonsh_env__.get("IGNOREEOF"):
@@ -169,6 +172,9 @@ class PromptToolkitShell(BaseShell):
         except Exception:  # pylint: disable=broad-except
             print_exception()
         toks = partial_color_tokenize(p)
+        if self._first_prompt:
+            carriage_return()
+            self._first_prompt = False
         self.settitle()
         return toks
 
